/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ComposedPhysicalObject.h: interface for the ComposedPhysicalObject class.
//
//////////////////////////////////////////////////////////////////////

#ifndef COMPOSEDPHYSICALOBJECT_H
#define COMPOSEDPHYSICALOBJECT_H

#include "PhysicalObject.h"

/** A composed object can consist of several physical objects (e.g., 
several spheres along a	line segment) that are rigidly bind together.

A composed object has a reference point that is not necessarily its center
of mass. The r, v parameters refer to the position of the reference 
point relative to the laboratory reference system (LRS). m is the total 
mass of the composed object. alpha, omega, I are properties 
of the rotation around the reference point.

The composed object owns the member objects (will destroy them on 
destruction).
*/

class ComposedPhysicalObject : public PhysicalObject  {
public:
	ComposedPhysicalObject(std::string label="");
	ComposedPhysicalObject(real x, real y, real alpha=0, std::string label="");

	/** Deletes member objects. */
	virtual ~ComposedPhysicalObject();


	/** Position of the center of mass relative to the reference point 
	(in the object reference system, ORS). */
	Vector2 s;

	/** Adds a previously created object to the objects composing the 
	link / composed object,
	also computes (updates) the mass properties.
	\param object is a previously created object
	\param x and
	\param y specifies the position of the center of mass of the added object 
	relative to the composed object reference system (ORS)
	\param alpha specifies the (fixed) rotation angle of the added object 
	relative to the ORS
	*/
	void addObject(PhysicalObject* object, real x, real y, real alpha);	

	/** Deletes member objects and empties the corresponding list. */
	void deleteObjects();

	/** Computes s, the position of the center of mass of the composed 
	object; the total mass of the composed object; and the moment of 
	inertia around its reference point. */
	void computeMassProperties();

	/** Computes the absolute positions and angles of the member objects 
	(positions relative to the LRS) based on the position of the 
	ORS relative to LRS and the positions of the member objects 
	relative to the ORS. Also computes the bounding box. */
	void computeMemberPositions();

	/** Advances the time to the next timestep. */
	virtual void integrate(const Integrator &integrator){
		PhysicalObject::integrate(integrator);
		computeMemberPositions();
	}

	/** Rolls back the time to the previous timestep. */
	virtual void rollback(){
		PhysicalObject::rollback();
		computeMemberPositions();
	}	

	virtual void registerPrimitives(Simulator* simulator);

	virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts);
	
	/** Detects whether a click of the mouse at rMouse has touched the object. In case of 
	contact, returns true and sets p to the relative vector between the center of the composed object
	and the contact point; p is expressed in the composed object reference frame because it rotates 
	with the object	while the object is dragged. */
	virtual bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object);

	virtual void setSensors();
	virtual void resetSensors();

	/** Calls the draw method for the member objects. */
	void draw(GUI *gui);

	void setOutlineColor(Color color);
	void setFillColor(Color color);

	
	/** The list of member objects. The list owns the member 
	objects.*/
	PhysicalObjectPVector objects;


};

#endif //COMPOSEDPHYSICALOBJECT_H
