/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef PHYSICALOBJECT_H
#define PHYSICALOBJECT_H

#include "GraphicalObject.h"
#include "ContactInfo.h"
#include "GlobalContactInfo.h"
#include "ContactSolver.h"
#include "ThyrixParameters.h"
#include <string>

class Border;
class Circle;
class CappedRectangle;
class Simulator;
class Integrator;

/** A physical object has the following properties: position r, velocity v, 
mass m, angle alpha, angular velocity omega. The linear properties
are the properties of the center of mass, relative to the laboratory reference system. The rotational
properties refer to the rotation around the z axis that passes through the center of mass of the object,
relative to the laboratory reference system. 

The object can have tactile sensors distributed on its surface. nSensors is the number of tactile 
sensors. The tactile sensors have a maximum activation (1) for a force greater than saturationForce.*/

class PhysicalObject : public GraphicalObject {
public:
	/** Default constructor; initializes everything with 0. */
	PhysicalObject(std::string label="", int nSensors=0, real saturationForce=0.5,
		Color outlineColor = GUI::colorBlack, Color fillColor = GUI::colorTransparent);
	
	/** Destructor; purges the lists. */
	virtual ~PhysicalObject(); 

	/** Registers all composing primitives of the object with the simulator. */
	virtual void registerPrimitives(Simulator* simulator);

	/** Contact handling. */
	virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) = 0;

  virtual bool detectContacts(Border* object, GlobalContactInfoVector* contacts) {
      return detectContacts((PhysicalObject*)object, contacts);
  }
  virtual bool detectContacts(Circle* object, GlobalContactInfoVector* contacts) { 
      return detectContacts((PhysicalObject*)object, contacts);
  }
  virtual bool detectContacts(CappedRectangle* object, GlobalContactInfoVector* contacts) { 
      return detectContacts((PhysicalObject*)object, contacts);
  }

	/** Sets the position r of the object. */
	void setPosition (real x, real y) { r.x=x; r.y=y; }

	/** Sets the velocity v of the object. */
	void setVelocity (real vx, real vy) { v.x=vx; v.y=vy; }

	/** Sets the mass m of the object. */
	void setMass (real m) { this->m=m; }

	/** Sets the mass based on the density. To be overridden.*/
	virtual void computeMass(real density = ThyrixParameters::defaultDensity) { m=1.0;}

	/** Computes the moment of inertia around the z axis that passes through the center,
	based on the mass and the geometrical properties. To be overridden. */
	virtual void computeInertia() { I=1.0; }

	/** Verifies if the object has circular symmetry; in this case, computations of rotational 
	properties can be skipped, on various occasions. By default, returns 0 (not symmetrical). 
	To be overridden by symmetrical objects (Circle). */
	virtual int isCircle() { return 0; }

	/** Normalizes Alpha to a value between 0 and 2 PI. */
	inline void normalizeAlpha();

	/** Advances the time to the next timestep. */
	virtual void integrate(const Integrator &integrator);

	/** Rolls back the time to the previous timestep. */
	virtual void rollback();

	/** Computes the coordinates of the bounding box. It is called after each call to integrate() or
	rollback(). */
	virtual void computeBox() {}

	/** Computes the derivatives (velocity, angular velocity) of the object, ignoring any eventual contact
	forces. */
	virtual void computeDerivativesWithoutContacts(ContactSolver* contactSolver);
	
	/** Computes the derivatives, given the contact forces. The globalContacts pointer is needed
	for contact processing for articulated agents. */
	virtual void computeDerivatives(GlobalContactInfoVector* globalContacts);

	/** Sets the activations of the tactile sensors, on the basis of the contact forces related to the 
	contacts of the object with other objects. */
	virtual void setSensors();

	/** Sets the activation of a tactile sensor, corresponding to the given contact. To be overridden. */
	virtual void setSensor(ContactInfo* contact) {}

	/** Transmits perceptual information to the controller and gets motor information from it.
	To be overridden by agents or other objects with self-generated movement.*/
	virtual void controll() {}

	/** Detects internal contacts. Does nothing for primitive objects or composed objects. To be 
	overriden by complex objects that have parts that move relative to each other, such as articulated
	objects. */
	virtual void detectInternalContacts(GlobalContactInfoVector* globalContacts) {}

	/** Delete the contacts of the object.*/
	virtual void deleteContacts();

	/** Resets tactile sensors to zero. */
	virtual void resetSensors();

	/** Adds given contact to the contact list.*/
	void addContact(ContactInfo* contact){
		contacts.push_back(contact);
	}

	/** Computes the contribution of the current object to the matrix of interactions between all objects
	that are in contact. */
	virtual void fillContactMatrix(ContactSolver* contactSolver, ContactInfo* contact);

	/** Draws the contact forces. */
	void drawContactForces(GUI* gui);

	/** Detects whether a click of the mouse at rMouse has touched the object. In case of 
	contact, returns true and sets p to the relative vector between the center of the object
	and the contact point; p is expressed in the object reference frame because it rotates with the object
	while the object is dragged. The object needs to be set by the function because, for articulated objects,
	the dragged object is an articulation and not the whole articulated object.
	To be overriden by the various object primitives. */
	virtual bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object) { return false; }


	/** Position of the center of mass of the object relative to laboratory reference system (LRS) */
	Vector2 r;

	/** Value of r at the previous timestep.*/
	Vector2 rOld;

	/** Velocity relative to the LRS */
	Vector2 v;
	
	/** Mass */
	real m;
 
	/** Rotation angle around center of mass relative to the LRS */
	real alpha;

	/** Value of alpha at the previous timestep. */
	real alphaOld;

	/** Angular velocity around center of mass relative to the LRS */
	real omega;

	/** Moment of inertia for rotation around the z axis that passes through the center of mass. */
	real I;

	/** The sum of external forces acting on the object */
	Vector2 externalForce;

	/** The sum of external torques acting on the object */
	real externalTorque;

	/** The parent of the object. It is NULL for simple objects, not NULL for objects that are 
	members of composed objects. */
	PhysicalObject* parent;

	/** The position of the member object relative to the reference system
	of the composed object (ORS), for objects that are members of composed
	objects.  */
	Vector2 relativeR;

	/** The rotation of the member object relative to the reference system
	of the composed object (ORS), for objects that are members of composed
	objects.  */
	real relativeAlpha;

	/** Minimum coordinates of the axis-aligned bounding box surrounding the object. */
	Vector2 boxMin;
	
	/** Minimum coordinates of the axis-aligned bounding box surrounding the object. */
	Vector2 boxMax;

	/** The list of contacts that act on the object. The list owns the contacts. */
	ContactInfoPVector contacts;

	/** A text label for the object; used primarily for debugging. */
	std::string label;

	/** A vector containing the activations of the sensors. */
	real* activations;

	/** The saturation contact force for contact sensors */
	real saturationForce;
	
protected:
	/** The number of unit sensors (pixels) in the tactile sensor. */
	int nSensors;
};

void PhysicalObject::normalizeAlpha() { 
	assert(alpha >-100 && alpha<100);
	while(alpha>2*M_PI) 
        alpha-=2*M_PI;
    while(alpha<0.0)
        alpha+=2*M_PI;
}

typedef std::vector<PhysicalObject*> PhysicalObjectPVector;

#endif //PHYSICALOBJECT_H
