/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef SIMULATOR_H
#define SIMULATOR_H

#include "World.h"
#include "Integrator.h"
#include "GlobalContactInfo.h"
#include "PhysicalObject.h"
#include "ContactSolver.h"
#include <set>

/** The main class of the Thyrix simulator. */

class Simulator : public World {
public:
	Simulator(); 
	virtual ~Simulator();

	virtual double getDt() const { return (double)integrator.getDt(); }
	
	/** Performs a timestep. It needs to be virtual because of the use of abstract Simulator classes in 
	GUI-related base classes. This method may be overridden to include fitness computing, agent tracking etc.
	*/
	virtual void advanceTime();

	/** Adds the object to the objects array. */
	void registerObject(PhysicalObject* object);

	/** Adds the object to the primitives array. */
	void registerPrimitive(PhysicalObject* object);

	/** Needs to be called after all objects were registered by the simulator, if we use
	box sorting for broad phase contact detection. */
	void setup(){}


	/** Indexes the contacts, resets to zero the contact data structures, 
	allocates memory for them if needed. */
	void indexContacts();

	/** Computes the matrix used for contact force computation. */
	void fillContactMatrix();

	virtual void detectContacts();

	void computeContacts();

	virtual void draw(GUI *gui);

	//Processing of user input: the user may apply forces to objects in the simulator,
	//by clicking and dragging with the mouse. 

	/** The object dragged by the mouse. */
	PhysicalObject* draggedObject;
	Vector2 draggingPoint;

	void setMouseForce(float x, float y);

	void unsetMouseForce();

	
	virtual void onMouseLeftDown(float x, float y){
		World::onMouseLeftDown(x,y);
		setMouseForce(x,y);
	}

	virtual void onMouseLeftUp(float x, float y){
		World::onMouseLeftUp(x,y);
		unsetMouseForce();
	}
	

	/** Adds the mouse force to the external forces (and torques) of the object dragged with the mouse. */
	void applyMouseForce();

	ContactSolver* contactSolver;
	
	/** The list of top level objects in the environment (borders, ball, agents, etc.). 
	This list owns the objects. 
	The top level objects may contain member objects, that they own. */
	PhysicalObjectPVector objects;

	/** The list of low level objects (primitives) in the environment. The list does not own the objects. 
	Currently, not used. */
	PhysicalObjectPVector primitives;

	/** The global list of contacts. This is a list of objects, not a list of 
	pointers.*/
	GlobalContactInfoVector contacts;

protected:

	Integrator integrator;

	virtual void controll();

	real realTime;

private:
	/** Deletes the object list and the included objects. */
	void deleteObjects();

	/** Deletes all previous contact information: 
  the global vector of contacts, and the contacts of all objects. */
	void deleteContacts();

	bool isContact;

	/** Lists of objects used for contact detection. The 2 lists correspond to the x and respectively y 
	coordinates used for sorting. */
	PhysicalObjectPVector sortedObjects[2];
	/** Sorts the n=0,1 list of sorted objects, with insertion sort. */
	void sortObjects(int n);

	/** A pair of objects: used for contact detection. */
	class ObjectPair {
		public:
		ObjectPair() { o1=o2=NULL;	}
		ObjectPair(PhysicalObject* no1, PhysicalObject* no2){
			//the pair is kept ordered in order to be unique
			if(no1<no2)	{o1=no1; o2=no2;} else {o2=no1; o1=no2;}
		}
		PhysicalObject* o1;
		PhysicalObject* o2;
		bool operator<(const ObjectPair& p2) const{
			if(o1<p2.o1)return true;
			else if(o1==p2.o1) return o2<p2.o2;
			else return false;
		}
	};

	std::set<ObjectPair> xContacts, xyContacts;


};

#endif //SIMULATOR_H
