/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/


/*

	How the agent is built
	TODO: This comment should be updated!

	- the agent has a circular body; radius and position are initialized, other properties 
	are set to 0 by default; mass and inertia are set based on default density;
	- two limbs are defined; for each limb is defined the angle theta and the distance l of the
	position of first joint, relative to the body;
	- we add links to the limbs:
		- we create the link; by default, theta0Min is -90 degrees, theta0Max is 90 degrees, 
		everything else is set to 0 on link creation;
		- we set theta0, theta as theta0, l;
		- we create an object and set its dimensions; mass and inertia are set based on default 
		density, other properties are 0;
		- we add the object to the link, by setting its position and angle relative to the link
		- we add the link to the limb; this also computes the total mass, center of mass and 
		inertia of the link, relative to the link point of reference; and computes IStar0; IStar 
		is set to IStar0.
		- at this moment, the absolute positions, velocities etc. of the link and objects is 0; we
		call forwardKinematics() to set these properties based on the absolute position of the 
		body,	and the theta angles, thetaD's etc. of the links


*/

#include "ArticulatedAgentBase.h"
#include "Simulator.h"
#include "GUI.h"
#include "purgeContainer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

    
ArticulatedAgentBase::ArticulatedAgentBase(std::string label) :
    ArticulatedComponent(label)
{
	nContacts=0;
}


ArticulatedAgentBase::~ArticulatedAgentBase(){
	deleteLimbs();
}


ArticulatedLimb* ArticulatedAgentBase::addLimb(){
    ArticulatedLimb* limb = new ArticulatedLimb(0, 0);
	limbs.push_back(limb);

	int i=limbs.size();
	char s[10];
	sprintf(s,"%d",i);
	limb->label=label+": limb "+s;

	return limb;
}

ArticulatedLimb* ArticulatedAgentBase::addLimb(real l, real theta){
	ArticulatedLimb* limb;
	limb=addLimb();
	limb->setProperties(l, theta);
	return limb;
}

void ArticulatedAgentBase::registerPrimitives(Simulator* simulator){
	//register body components
	ComposedPhysicalObject::registerPrimitives(simulator);
	for(ArticulatedLimbPVector::iterator limbsI = limbs.begin(), 
            limbsEnd = limbs.end(); limbsI != limbsEnd; ++limbsI){
        ArticulatedLimb* limb = *limbsI;
		for(ArticulatedLinkPVector::iterator linksI = limb->links.begin(), 
                linksEnd = limb->links.end(); linksI != linksEnd; ++linksI){
			//register links components
			(*linksI)->registerPrimitives(simulator);
		}
	}
}

void ArticulatedAgentBase::solveSystem(SymmetricMatrix3& m, const Vector3& c, Vector3& x){
	real determinant;
	determinant=m.getDeterminant();
	m.mirror();
	solveSystem(m, determinant, c, x);
}

void ArticulatedAgentBase::solveSystem(const SymmetricMatrix3& m, const real determinant, 
                                       const Vector3& c, Vector3& x){
	/*
	m is a symmetric matrix. When we change a column, it may not remain symmetrical. mm is not symmetrical,
	and we should complete the lower half of m ("mirror") before assigning it to mm.
	*/
	Matrix3 mm;
	assert(determinant!=0.0);
	for(int i=1; i<=3; i++){
		mm=m;
		mm.setColumn(i,c);
		x.setElement(i,mm.getDeterminant()/determinant);
	}
}

bool ArticulatedAgentBase::detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts){
	bool isContact=false;
	isContact = isContact || ComposedPhysicalObject::detectContacts(object, contacts);
	for(ArticulatedLimbPVector::iterator limbsI = limbs.begin(), 
            limbsEnd = limbs.end(); limbsI != limbsEnd; ++limbsI){
        ArticulatedLimb* limb = *limbsI;
		for(ArticulatedLinkPVector::iterator linksI = limb->links.begin(), 
                linksEnd = limb->links.end(); linksI != linksEnd; ++linksI){
			isContact = (*linksI)->detectContacts(object, contacts) || isContact;
		}
	}
	return isContact;
}

void ArticulatedAgentBase::detectInternalContacts(GlobalContactInfoVector* contacts){
	ArticulatedLimb* limb1;
	ArticulatedLimb* limb2;
	ArticulatedLimbPVector::iterator limbsI1, limbsI2, limbsEnd;
	ArticulatedLinkPVector::iterator linksI1, linksEnd1, linksI2, linksEnd2;
	
	for(limbsI1=limbs.begin(), limbsEnd=limbs.end(); limbsI1!=limbsEnd; limbsI1++){
		limb1=*limbsI1;
		for (linksI1=limb1->links.begin(), linksEnd1=limb1->links.end(); linksI1!=linksEnd1; linksI1++){
			/* Link-body contact; we don't have contact between the body and the first link. */
			if(linksI1!=limb1->links.begin()) {
				ComposedPhysicalObject::detectContacts(*linksI1, contacts);
			}
			// Torque contact
			(*linksI1)->detectTorqueContact(contacts);
			// Link-link contact with other limbs
			limbsI2=limbsI1;
			limbsI2++;
			if(limbsI2!=limbsEnd){				
				for( ; limbsI2!=limbsEnd; limbsI2++){	
					limb2=*limbsI2;
					for (linksI2=limb2->links.begin(), linksEnd2=limb2->links.end(); linksI2!=linksEnd2; linksI2++){
						(*linksI2)->detectContacts(*linksI1, contacts);
					}
				}
			}
			/* Link-link contact on the same limb; we don't have contacts between succesive links. 
			Assumes that the parent link appears earlier in the link list. */
			linksI2=linksI1;
			linksI2++;
			if(linksI2!=linksEnd1){
				for (; linksI2!=linksEnd1; linksI2++){
					if((*linksI1)!=(*linksI2)->parentLink)
						(*linksI1)->detectContacts(*linksI2, contacts);
				}
			}
		}
	}
}

bool ArticulatedAgentBase::detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object)	{
	bool isContact=false;
	isContact = ComposedPhysicalObject::detectMouseContact(rMouse, p, object);
	for(ArticulatedLimbPVector::iterator limbsI = limbs.begin(), 
            limbsEnd = limbs.end(); limbsI != limbsEnd && !isContact; ++limbsI){
        ArticulatedLimb* limb = *limbsI;
		for(ArticulatedLinkPVector::iterator linksI = limb->links.begin(), 
                linksEnd = limb->links.end(); linksI != linksEnd && !isContact; ++linksI){
			isContact = (*linksI)->detectMouseContact(rMouse, p, object);
		}
	}
	return isContact;
}

void ArticulatedAgentBase::deleteContacts(){
	ArticulatedLimb* limb;
	ArticulatedLimbPVector::iterator limbsI, limbsEnd;
	ArticulatedLinkPVector::iterator linksI, linksEnd;
	ComposedPhysicalObject::deleteContacts();
	for(limbsI=limbs.begin(), limbsEnd=limbs.end(); limbsI!=limbsEnd; limbsI++){
		limb=*limbsI;
		for(linksI=limb->links.begin(), linksEnd=limb->links.end(); linksI!=linksEnd; linksI++){
			(*linksI)->deleteContacts();
		}
	}
}

void ArticulatedAgentBase::fillContactMatrix(ContactSolver* contactSolver, ContactInfo* contactInfo){
	/*
	contactInfo is the info of contact alpha relative to the current object
	Assumes v_0, w_0 (the velocity without the contribution of the contacts) are
	already computed as v, omega. 
	*/
	Vector2 vt1, vt2;
	for(LinkContactInfoVector::iterator Qi=Q.begin(), Qend=Q.end(); Qi!=Qend; ++Qi){
		vt1.setXY(Qi->v.y,Qi->v.z);
		vt1.rotate(alpha);
		vt2=contactInfo->p;
		vt2.cross(Qi->v.x);
		vt1+=vt2;
		contactSolver->contactMatrix[contactInfo->alpha][Qi->alpha]+=
			vt1*contactInfo->n*contactInfo->sigma;		
	}
	vt1=contactInfo->p;
	vt1.cross(omega);
	vt1+=v;
	contactSolver->contactVector[contactInfo->alpha]+=vt1*contactInfo->n*contactInfo->sigma;
}

void ArticulatedAgentBase::deleteLimbs(){
	purgeContainer(limbs);
}

void ArticulatedAgentBase::draw(GUI *gui){
	ComposedPhysicalObject::draw(gui);
	for(ArticulatedLimbPVector::iterator it = limbs.begin(), end=limbs.end();
        it != end; ++it) {
            (*it)->draw(gui);
    }
}

void ArticulatedAgentBase::setOutlineColor(Color color){
	ComposedPhysicalObject::setOutlineColor(color);
	for(ArticulatedLimbPVector::iterator it = limbs.begin(), end=limbs.end();
        it != end; ++it) {
            (*it)->setOutlineColor(color);
    }
}

void ArticulatedAgentBase::setFillColor(Color color){
	ComposedPhysicalObject::setFillColor(color);
	for(ArticulatedLimbPVector::iterator it = limbs.begin(), end=limbs.end();
        it != end; ++it) {
            (*it)->setFillColor(color);
    }
}
