/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedLink.cpp: implementation of the ArticulatedLink class.
//
//////////////////////////////////////////////////////////////////////

#include "ArticulatedLink.h"
#include "ArticulatedLimb.h"
#include "Simulator.h"
#include "Integrator.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ArticulatedLink::ArticulatedLink(const std::string &label) :
    ArticulatedComponent(label){
	parentLink=NULL;
	theta=0.0;
	thetaD=0.0;
	thetaDD=0.0;
	deltaThetaMax=90*degrees;
	theta0=0.0;
	theta0Min=-80.0*degrees;
	theta0Max=80.0*degrees;
	k=DEFAULT_K;
	eta=DEFAULT_ETA;
	torqueContact=NULL;
	motorTorque=0;
}

ArticulatedLink::~ArticulatedLink(){
}

void ArticulatedLink::integrate(const Integrator &integrator){ 
    thetaOld = theta;
    integrator.integrate(theta, thetaD);
}

void ArticulatedLink::normalizeTheta(){
	while(theta-theta0>M_PI)
		theta-=2*M_PI;
	while(theta-theta0<-M_PI)
		theta+=2*M_PI;
}

void ArticulatedLink::computeMotorTorque(){
	if(hasElasticTorque){
		//normalizeTheta();
		motorTorque=-k*(theta-theta0);//-eta*thetaD;
	}
}


void ArticulatedLink::computeSinCos(){
	sinTheta=sin(theta);
	cosTheta=cos(theta);
	sin2=sinTheta*sinTheta;
	cos2=cosTheta*cosTheta;
	sinCos=sinTheta*cosTheta;
}

void ArticulatedLink::setParentLink(ArticulatedLink* parentLink){
	this->parentLink=parentLink;
	if(parentLink!=NULL){
		parentLink->childLinks.push_back(this);
	}
}


void ArticulatedLink::computeForceQuasistatic(GlobalContactInfoVector* globalContacts){
	Vector2 vt2;
	LinkContactInfoPVector::iterator Ki, Kend;
	LinkContactInfoVector::iterator Qi, Qend;

	force.setToZero();
	Ki=K.begin();
	Kend=K.end();

	for (Qi=Q.begin(), Qend=Q.end(); Qi!=Qend; Qi++){
		Vector3& vt=Qi->v;
		vt2.x=IStar.getElement(1,2)*vt.x+IStar.getElement(2,2)*vt.y+
							IStar.getElement(2,3)*vt.z;
		vt2.y=IStar.getElement(1,3)*vt.x+IStar.getElement(2,3)*vt.y+
							IStar.getElement(3,3)*vt.z;
		//advance K until K.alpha>=alpha
		while(Ki!=Kend && (*Ki)->alpha<Qi->alpha)
			Ki++;
		if(Ki!=Kend && (*Ki)->alpha==Qi->alpha){
			vt2.x+=(*Ki)->v.y;
			vt2.y+=(*Ki)->v.z;
		}
		vt2*=(*globalContacts)[Qi->alpha].force;
		force+=vt2;
	}
	force.rotate(alpha);
}


void ArticulatedLink::detectTorqueContact(GlobalContactInfoVector* contacts){
	if(theta>theta0Max || theta<theta0Min){
        int sigma = (theta < theta0Min) ? 1 : -1;
		torqueContact=new ContactInfo(this, sigma);	
		contacts->push_back(GlobalContactInfo(torqueContact, NULL, (real)0.0));
	}	
}

void ArticulatedLink::draw(GUI *gui){
	//draws the composing object
	ComposedPhysicalObject::draw(gui);
	//draws itself
	
	real cosAlpha=cos(alpha);
	real sinAlpha=sin(alpha);
	
	//the link point of reference
	gui->setPenColor(outlineColor);
	gui->drawCircle(r.x, r.y, 0.02f);

	//the link itself
	//gui->drawLine(r.x, r.y, r.x+l*cosAlpha, r.y+l*sinAlpha);
	
	/*
	//a line at the target angle, theta0
	p->dc->SetPen(*wxMEDIUM_GREY_PEN);
	p->dc->DrawLine(
		p->panX+p->zoomX*(r.x),
		p->panY+p->zoomY*(r.y), 
		p->panX+p->zoomX*(r.x+l*cos(alpha+theta0-theta)),
		p->panY+p->zoomY*(r.y+l*sin(alpha+theta0-theta)) );
	*/

	/*
	real ll; 
	ll=l;
	if(ll<0.1) ll=0.1;
	//illustrate torque contacts
	if(torqueContact!=NULL){
		//the link itself
		//gui->setPenColor(GUI::GREEN);
		//gui->drawLine(r.x, r.y, r.x+ll*cosAlpha, r.y+ll*sinAlpha);
		//if contact, draw min and max theta
		gui->setPenColor(GUI::RED);
		if(torqueContact->sigma<0){
			gui->drawLine(r.x, r.y, r.x+ll*cos(alpha+theta0Max-theta),
					r.y+ll*sin(alpha+theta0Max-theta));
			gui->drawTorque(r,alpha+theta0Max-theta,torqueContact->force*torqueContact->sigma);
		} else {
			gui->drawLine(r.x, r.y, r.x+ll*cos(alpha+theta0Min-theta),
					r.y+ll*sin(alpha+theta0Min-theta) );
			gui->drawTorque(r,alpha+theta0Min-theta,torqueContact->force*torqueContact->sigma);
		}	
		
	}
	*/
	
	/*
	//the total force
	gui->setPenColor(Color("red"));
	Vector2 v;
	v=s;
	v.rotate(alpha);
	v+=r;
	gui->drawForce(v,force);
	*/
	
}
